﻿using combit.ListLabel25.Repository;
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Xml.Serialization;

namespace llrepotest
{

    public class DirectoryBasedRepositoryItem : RepositoryItem
    {

        public string CustomData { get; set; }

        public DirectoryBasedRepositoryItem(string internalID, string descriptor, string type, DateTime lastModificationUTC)
            : base(internalID, descriptor, type, lastModificationUTC)
        { }

        public DirectoryBasedRepositoryItem()  // parameterless constructor for serializer
            : base(null, null, null, DateTime.MinValue)
        { }
    }

    public class DirectoryBasedFileRepository : IRepository
    {

        private readonly string _baseDirectory;
        private readonly XmlSerializer _xmlSerializer;
        private readonly Dictionary<string, DirectoryBasedRepositoryItem> _cachedItems;

        public string rootrepoitem;

        public DirectoryBasedFileRepository(string baseDirectory)
        {
            _baseDirectory = baseDirectory;
            _cachedItems = new Dictionary<string, DirectoryBasedRepositoryItem>();
            _xmlSerializer = new XmlSerializer(typeof(DirectoryBasedRepositoryItem));

            if (!Directory.Exists(_baseDirectory))
            {
                Directory.CreateDirectory(_baseDirectory);
            }
            else
            {
                foreach (var metadataFile in Directory.GetFiles(_baseDirectory, "*.repinfo"))
                {
                    var itemMetadata = ReadMetadataFromFile(metadataFile);

                    _cachedItems.Add(itemMetadata.InternalID, itemMetadata);
                }
            }
        }


        private string GetMetadataFile(string id)
        {
            return Path.Combine(_baseDirectory, id.GetHashCode() + ".repinfo");
        }

        private string GetPayloadFile(string id)
        {
            return Path.Combine(_baseDirectory, id.GetHashCode() + ".repdata");
        }


        public void DeleteItem(string internalID)
        {
            _cachedItems.Remove(internalID);
            File.Delete(GetMetadataFile(internalID));
            File.Delete(GetPayloadFile(internalID));
        }

        public IEnumerable<RepositoryItem> GetAllItems()
        {
            return _cachedItems.Values;
        }

     
        private DirectoryBasedRepositoryItem ReadMetadataFromFile(string metadataFile)
        {
            using (var fs = new FileStream(metadataFile, FileMode.Open, FileAccess.Read, FileShare.Read))
            {
                var item = (DirectoryBasedRepositoryItem)_xmlSerializer.Deserialize(fs);
                item.IsEmpty = !File.Exists(GetPayloadFile(item.InternalID));
                return item;
            }
        }

        public void LoadItem(string internalID, Stream destinationStream, CancellationToken cancelToken)
        {
            if (!File.Exists(GetPayloadFile(internalID)))   // No content file might exist yet if this was a newly created repository item
                return;

            using (var dataFile = new FileStream(GetPayloadFile(internalID), FileMode.Open, FileAccess.Read, FileShare.None))
            {
                dataFile.CopyTo(destinationStream);
            }
        }

        public void CreateOrUpdateItem(RepositoryItem updatedItem, string userImportData, Stream sourceStream)
        {
            // Create or update the metadata file
            DirectoryBasedRepositoryItem item;
            if (ContainsItem(updatedItem.InternalID))
            {
                item = _cachedItems[updatedItem.InternalID];
                item.Descriptor = updatedItem.Descriptor;
                item.LastModificationUTC = DateTime.UtcNow;
                item.IsEmpty = false;
            }
            else
            {
                item = new DirectoryBasedRepositoryItem(updatedItem.InternalID, updatedItem.Descriptor, updatedItem.Type, DateTime.UtcNow) { CustomData = userImportData };
                _cachedItems.Add(item.InternalID, item);
            }

            // Save file content
            if (sourceStream != null)   // Stream is null, when a new repository item is created by code and was not yet saved in the designer
            {
                using (var payloadFile = new FileStream(GetPayloadFile(updatedItem.InternalID), FileMode.Create, FileAccess.ReadWrite, FileShare.None))
                {
                    sourceStream.CopyTo(payloadFile);
                }
            }

            //  Save file metadata
            using (var metadataFile = new FileStream(GetMetadataFile(updatedItem.InternalID), FileMode.Create, FileAccess.ReadWrite, FileShare.None))
            {
                _xmlSerializer.Serialize(metadataFile, item);
            }
        }

        public bool ContainsItem(string internalID)
        {
            return _cachedItems.ContainsKey(internalID);
        }

        public RepositoryItem GetItem(string internalID)
        {
            return _cachedItems[internalID];
        }

        public void SetDescriptor(string internalID, string value)
        {
            var item = _cachedItems[internalID];
            item.Descriptor = value;
            item.LastModificationUTC = DateTime.UtcNow;

            using (var metadataFile = new FileStream(GetMetadataFile(internalID), FileMode.Create, FileAccess.ReadWrite, FileShare.None))
            {
                _xmlSerializer.Serialize(metadataFile, item);
            }
        }

        public bool LockItem(string id)
        {
            // If required, a repository item can be locked when it is loaded for editing.
            // IMPORTANT: Always implement a fallback to release the locks (e.g. timeout). Especially when used with the Web Designer, UnlockItem() might not get called due to network problems.

            // Return true, if the lock was acquired or if no locking is implemented.
            // Return false, if the item is locked by an other user. The designer will show an error message or open the item in read-only mode.
            return true;
        }

        public void UnlockItem(string id)
        {
            // see LockItem()
        }
    }
}
